==> cbra-0.19.1.tar.gz: cbra-0.19.1/cbra/ext/google/googlejwtprincipal.py <==
"""Declares :class:`GoogleJWTPrincipal`."""
import typing

import fastapi
from fastapi.security import HTTPBearer
from unimatrix.ext.kms import JSONWebKeySet
from unimatrix.ext.kms import JSONWebToken

from cbra.lib.auth import IssuerJWKS
from cbra.lib.auth import JWTPrincipal
from .const import GOOGLE_OPENID_SERVER


class GoogleJWTPrincipal(JWTPrincipal):
    """A JSON Web Token (JWT) that is signed by Google."""
    __module__: str = 'cbra.ext.google'

    def __init__(self,
        audience: typing.Union[set, str, typing.Callable] = None
    ):
        self.audience = audience
        super().__init__()

    async def resolve(self,
        request: fastapi.Request,
        bearer: str = fastapi.Depends(HTTPBearer(auto_error=False)),
        jwks: JSONWebKeySet = IssuerJWKS([GOOGLE_OPENID_SERVER])
    ) -> JSONWebToken:
        """Parse the ``Authorization`` header (ensure that the scheme is
        ``Bearer``), proceed to decode a JSON Web Token (JWT), verify that
        it was signed by Google.

        Return a :class:`~unimatrix.ext.kms.JSONWebToken` on success, else
        raise a fatal exception (which is assumed to be recogized by the
        caller and converted into the appropriate error response).
        """
        if bearer is None: # pragma: no cover
            return None
        token = JSONWebToken.decode(bearer.credentials)
        await token.verify_signature(jwks)
        token.validate(
            audience=self.get_allowed_audiences(request),
            issuer=GOOGLE_OPENID_SERVER,
            required={'iat', 'exp'}
        )
        return token # pragma: no cover

    def get_allowed_audiences(self,
        request: fastapi.Request
    ) -> typing.Set[str]:
        """Return a set containing the allowed audiences for the presented
        token. If the `audience` parameter was not passed to the constructor,
        this method returns the top-level URL.
        """
        assert request.url.scheme is not None # nosec
        assert request.url.netloc is not None # nosec
        assert request.url.path is not None # nosec
        if self.audience is None:
            return {f'{request.url.scheme}://{request.url.netloc}'}

        if callable(self.audience):
            audience = self.audience(request)
        else:
            audience = self.audience

        return audience if isinstance(audience, set) else {audience}

==> cbra-0.50.0.tar.gz: cbra-0.50.0/cbra/ext/google/googlejwtprincipal.py <==
"""Declares :class:`GoogleJWTPrincipal`."""
import inspect
import typing

import fastapi
from fastapi.security import HTTPBearer
from unimatrix.ext.kms import JSONWebKeySet
from unimatrix.ext.kms import JSONWebToken

from cbra.lib.auth import JWTPrincipal
from cbra.lib.auth import IssuerJWKS
from cbra.lib.auth import NullJWKS
from .const import GOOGLE_OPENID_SERVER
from .jwks import GoogleJWKS


class GoogleJWTPrincipal(JWTPrincipal):
    """A JSON Web Token (JWT) that is signed by Google."""
    __module__: str = 'cbra.ext.google'

    def __init__(self,
        audience: typing.Union[set, str, typing.Callable] = None,
        jwks: IssuerJWKS = GoogleJWKS
    ):
        self.audience = audience
        self.jwks = jwks
        super().__init__()

    def get_signature(self) -> inspect.Signature:
        """Return a :class:`inspect.Signature` instance representing the
        call sigature of the dependency resolver.
        """
        # Annotate the `jwks` argument to hold the actual instance passed
        # to the constructor.
        signature = super().get_signature()
        params = [
            x for x in list(signature.parameters.values())
            if x.name != 'jwks'
        ]
        params.append(
            inspect.Parameter(
                name='jwks',
                kind=inspect.Parameter.KEYWORD_ONLY,
                annotation=IssuerJWKS,
                default=self.jwks
            )
        )
        return inspect.Signature(params)

    async def resolve(self,
        request: fastapi.Request,
        bearer: str = fastapi.Depends(HTTPBearer(auto_error=False)),
        jwks: JSONWebKeySet = NullJWKS()
    ) -> JSONWebToken:
        """Parse the ``Authorization`` header (ensure that the scheme is
        ``Bearer``), proceed to decode a JSON Web Token (JWT), verify that
        it was signed by Google.

        Return a :class:`~unimatrix.ext.kms.JSONWebToken` on success, else
        raise a fatal exception (which is assumed to be recogized by the
        caller and converted into the appropriate error response).
        """
        if bearer is None: # pragma: no cover
            return None
        token = JSONWebToken.decode(bearer.credentials)
        await token.verify_signature(jwks)
        token.validate(
            audience=self.get_allowed_audiences(request),
            issuer=GOOGLE_OPENID_SERVER,
            required={'iat', 'exp'}
        )
        return token # pragma: no cover

    def get_allowed_audiences(self,
        request: fastapi.Request
    ) -> typing.Set[str]:
        """Return a set containing the allowed audiences for the presented
        token. If the `audience` parameter was not passed to the constructor,
        this method returns the top-level URL.
        """
        assert request.url.scheme is not None # nosec
        assert request.url.netloc is not None # nosec
        assert request.url.path is not None # nosec
        if self.audience is None:
            return {f'{request.url.scheme}://{request.url.netloc}'}

        if callable(self.audience):
            audience = self.audience(request)
        else:
            audience = self.audience

        return audience if isinstance(audience, set) else {audience}
